=== modified file '__init__.py'
--- __init__.py	2006-05-18 14:19:48 +0000
+++ __init__.py	2006-05-18 15:48:22 +0000
@@ -370,12 +370,23 @@
         branch_mark(mark, branch, delete)
 
 class cmd_import(bzrlib.commands.Command):
-    takes_args = ['source']
-    def run(self, source):
+    """Import sources from a tarball <BZRTOOLS>
+    
+    This command will import a tarball into a bzr tree, replacing any versioned
+    files already present.  If a directory is specified, it is used as the
+    target.  If the directory does not exist, or is not versioned, it is
+    created.
+
+    Tarballs may be gzip or bzip2 compressed.  This is autodetected.
+
+    If the tarball has a single root directory, that directory is stripped
+    when extracting the tarball.
+    """
+    
+    takes_args = ['source', 'tree?']
+    def run(self, source, tree=None):
         from upstream_import import do_import
-        do_import(source)
-                
-
+        do_import(source, tree)
 
 
 commands = [cmd_shelve, cmd_unshelve, cmd_shelf, cmd_clean_tree,
@@ -384,8 +395,6 @@
             cmd_multi_pull, cmd_switch, cmd_branch_mark, cmd_import]
 
 
-
-
 import bzrlib.builtins
 commands.append(rspush.cmd_rspush)
 

=== modified file 'tests/blackbox.py'
--- tests/blackbox.py	2006-05-18 14:23:45 +0000
+++ tests/blackbox.py	2006-05-18 15:48:22 +0000
@@ -108,11 +108,20 @@
         self.runbzr('add')
         self.runbzr('commit -m hello')
         self.runbzr('export ../source-0.1.tar.gz')
-        os.mkdir('../import')
+        self.runbzr('init ../import')
         os.chdir('../import')
         self.runbzr('import ../source-0.1.tar.gz')
+        self.failUnlessExists('src/myfile')
         result = self.runbzr('import ../source-0.1.tar.gz', retcode=3)[1]
         self.assertContainsRe(result, 'Working tree has uncommitted changes')
+        self.runbzr('commit -m commit')
+        self.runbzr('import ../source-0.1.tar.gz')
+        os.chdir('..')
+        self.runbzr('init import2')
+        self.runbzr('import source-0.1.tar.gz import2')
+        self.failUnlessExists('import2/src/myfile')
+        self.runbzr('import source-0.1.tar.gz import3')
+        self.failUnlessExists('import3/src/myfile')
 
 
 def test_suite():

=== modified file 'upstream_import.py'
--- upstream_import.py	2006-05-18 14:33:35 +0000
+++ upstream_import.py	2006-05-18 15:48:22 +0000
@@ -8,8 +8,8 @@
 
 from bzrlib.bzrdir import BzrDir
 from bzrlib.delta import compare_trees
-from bzrlib.errors import NoSuchFile, BzrCommandError
-from bzrlib.osutils import pathjoin, isdir
+from bzrlib.errors import NoSuchFile, BzrCommandError, NotBranchError
+from bzrlib.osutils import pathjoin, isdir, file_iterator
 from bzrlib.tests import TestCaseInTempDir
 from bzrlib.transform import TreeTransform
 from bzrlib.workingtree import WorkingTree
@@ -61,7 +61,8 @@
         added.append(relative_path)
         path = tree.abspath(relative_path)
         if member.isreg():
-            tt.create_file([tar_file.extractfile(member).read()], trans_id)
+            tt.create_file(file_iterator(tar_file.extractfile(member)), 
+                           trans_id)
         elif member.isdir():
             if isdir(path):
                 tt.cancel_deletion(trans_id)
@@ -82,9 +83,18 @@
             tree.add(path)
 
 
-def do_import(source):
+def do_import(source, tree_directory=None):
     """Implementation of import command.  Intended for UI only"""
-    tree = WorkingTree.open_containing('.')[0]
+    if tree_directory is not None:
+        try:
+            tree = WorkingTree.open(tree_directory)
+        except NotBranchError:
+            if not os.path.exists(tree_directory):
+                os.mkdir(tree_directory)
+            branch = BzrDir.create_branch_convenience(tree_directory)
+            tree = branch.bzrdir.open_workingtree()
+    else:
+        tree = WorkingTree.open_containing('.')[0]
     tree.lock_write()
     try:
         if compare_trees(tree, tree.basis_tree()).has_changed():

